import React, { useEffect, useState } from "react";
import { useParams } from "react-router-dom";
import axios from "axios";
import styles from "./EditUserDetailsStyles.module.css";
import { useNavigate } from "react-router-dom";

const EditUserDetails = () => {
  const { personId } = useParams();

  // data to be stored from api
  const [personInfo, setPersonInfo] = useState(null);
  const [degrees, setDegrees] = useState([]);
  const [addresses, setAddresses] = useState([]);
  const [contacts, setContacts] = useState([]);
  const [involvements, setInvolvements] = useState([]);
  const [loading, setLoading] = useState(true);
  const [error, setError] = useState(null);

  const navigate = useNavigate();

  // I added these two useStates so we will be able to check for editing and deleting
  const [isEditing, setIsEditing] = useState(false);
  const [showDeleteConfirm, setShowDeleteConfirm] = useState(false);

  useEffect(() => {
    const fetchPersonDetails = async () => {
      setLoading(true);
      setError(null);

      try {
        // get the person's full details as soon as the page mounts
        const response = await axios.get(
          `${process.env.REACT_APP_API_URL}/person/fullInfo/${personId}`
        );

        console.log("Response data.person: ", response.data.person);

        const { personInfo, degrees, addresses, contacts, involvements } =
          response.data.person;

        // Set individual states
        setPersonInfo(personInfo);
        setDegrees(degrees);
        setAddresses(addresses);
        setContacts(contacts);
        setInvolvements(involvements);
      } catch (err) {
        setError(
          err.response?.data?.message || "Failed to fetch user details."
        );
      } finally {
        setLoading(false);
      }
    };

    fetchPersonDetails();
  }, [personId]);

  const handleEdit = () => {
    setIsEditing(true);
  };

  const addNewItem = (setter, template) => {
    setter((prev) => [...prev, template]);
  };

  const deleteItem = (setter, index) => {
    setter((prev) => prev.filter((_, i) => i !== index));
  };

  const handleSave = async () => {
    console.log("HandleSave");

    try {
      const updatedDetails = {
        personInfo,
        degrees,
        addresses,
        contacts,
        involvements,
      };

      console.log("Updated details: ", updatedDetails);

      await axios.put(
        `${process.env.REACT_APP_API_URL}/person/update/${personId}`,
        updatedDetails
      );
      setIsEditing(false);
      alert("User details updated successfully!");
    } catch (err) {
      alert("Failed to update user details.");
    }
  };

  const handleDelete = async () => {
    console.log("HandleDelete");

    try {
      await axios.delete(`${process.env.REACT_APP_API_URL}/person/${personId}`);
      alert("User deleted successfully!");
    } catch (err) {
      alert("Failed to delete user.");
    }

    navigate("/admin/search-people-edit");
  };

  if (loading) return <p>Loading user details...</p>;
  if (error) return <p className={styles.error}>{error}</p>;

  return (
    <div className={styles.container}>
      <h2>Edit User Details</h2>
      <div>
        <h3>Personal Information</h3>
        {Object.entries(personInfo || {}).map(([key, value]) =>
          key.toLowerCase().includes("id") ? null : ( // Exclude IDs
            <p key={key}>
              <strong>{key.replace(/([A-Z])/g, " $1")}:</strong>{" "}
              {isEditing ? (
                <input
                  type="text"
                  value={value || ""}
                  onChange={(e) =>
                    setPersonInfo((prev) => ({
                      ...prev,
                      [key]: e.target.value,
                    }))
                  }
                />
              ) : (
                value || "N/A"
              )}
            </p>
          )
        )}
      </div>

      <div>
        <h3>Degrees</h3>
        {degrees.map((degree, index) => (
          <div key={index} className={styles.listItem}>
            {Object.entries(degree).map(([key, value]) =>
              key.toLowerCase().includes("id") ? null : ( // Exclude IDs
                <p key={key}>
                  <strong>{key.replace(/([A-Z])/g, " $1")}:</strong>{" "}
                  {isEditing ? (
                    <input
                      type="text"
                      value={value || ""}
                      onChange={(e) =>
                        setDegrees((prev) =>
                          prev.map((deg, i) =>
                            i === index
                              ? { ...deg, [key]: e.target.value }
                              : deg
                          )
                        )
                      }
                    />
                  ) : (
                    value || "N/A"
                  )}
                </p>
              )
            )}
            {isEditing && (
              <button
                className={styles.removeButton}
                onClick={() => deleteItem(setDegrees, index)}
              >
                Remove
              </button>
            )}
          </div>
        ))}
        {isEditing && (
          <button
            className={styles.addButton}
            onClick={() =>
              addNewItem(setDegrees, {
                degreeType: "",
                degreeDepartment: "",
                degreeCollege: "",
                degreeYear: "",
                degreeDescription: "",
              })
            }
          >
            Add Degree
          </button>
        )}
      </div>

      <div>
        <h3>Addresses</h3>
        {addresses.map((address, index) => (
          <div key={index} className={styles.listItem}>
            {Object.entries(address).map(([key, value]) =>
              key.toLowerCase().includes("id") ? null : ( // Exclude IDs
                <p key={key}>
                  <strong>{key.replace(/([A-Z])/g, " $1")}:</strong>{" "}
                  {isEditing ? (
                    <input
                      type="text"
                      value={value || ""}
                      onChange={(e) =>
                        setAddresses((prev) =>
                          prev.map((addr, i) =>
                            i === index
                              ? { ...addr, [key]: e.target.value }
                              : addr
                          )
                        )
                      }
                    />
                  ) : (
                    value || "N/A"
                  )}
                </p>
              )
            )}
            {isEditing && (
              <button
                className={styles.removeButton}
                onClick={() => deleteItem(setAddresses, index)}
              >
                Remove
              </button>
            )}
          </div>
        ))}
        {isEditing && (
          <button
            className={styles.addButton}
            onClick={() =>
              addNewItem(setAddresses, {
                address1: "",
                address2: "",
                city: "",
                state: "",
                country: "",
                zipCode: "",
                preferredAddress: false,
              })
            }
          >
            Add Address
          </button>
        )}
      </div>

      <div>
        <h3>Contacts</h3>
        {contacts.map((contact, index) => (
          <div key={index} className={styles.listItem}>
            {Object.entries(contact).map(([key, value]) =>
              key.toLowerCase().includes("id") ? null : ( // Exclude IDs
                <p key={key}>
                  <strong>{key.replace(/([A-Z])/g, " $1")}:</strong>{" "}
                  {isEditing ? (
                    <input
                      type="text"
                      value={value || ""}
                      onChange={(e) =>
                        setContacts((prev) =>
                          prev.map((con, i) =>
                            i === index
                              ? { ...con, [key]: e.target.value }
                              : con
                          )
                        )
                      }
                    />
                  ) : (
                    value || "N/A"
                  )}
                </p>
              )
            )}
            {isEditing && (
              <button
                className={styles.removeButton}
                onClick={() => deleteItem(setContacts, index)}
              >
                Remove
              </button>
            )}
          </div>
        ))}
        {isEditing && (
          <button
            className={styles.addButton}
            onClick={() =>
              addNewItem(setContacts, {
                contactType: "",
                contactNumber: "",
                preferredContact: false,
              })
            }
          >
            Add Contact
          </button>
        )}
      </div>

      <div>
        <h3>Involvements</h3>
        {involvements.map((involvement, index) => (
          <div key={index} className={styles.listItem}>
            <p>
              {isEditing ? (
                <input
                  type="text"
                  value={involvement || ""}
                  onChange={(e) =>
                    setInvolvements((prev) =>
                      prev.map((inv, i) => (i === index ? e.target.value : inv))
                    )
                  }
                />
              ) : (
                involvement || "N/A"
              )}
            </p>
            {isEditing && (
              <button
                className={styles.removeButton}
                onClick={() => deleteItem(setInvolvements, index)}
              >
                Remove
              </button>
            )}
          </div>
        ))}
        {isEditing && (
          <button
            className={styles.addButton}
            onClick={() => addNewItem(setInvolvements, "")}
          >
            Add Involvement
          </button>
        )}
      </div>

      <div className={styles.actions}>
        {!isEditing ? (
          <button onClick={handleEdit}>Edit</button>
        ) : (
          <button onClick={handleSave}>Save</button>
        )}
        <button onClick={() => setShowDeleteConfirm(true)}>Delete</button>
      </div>

      {showDeleteConfirm && (
        <div className={styles.confirmPopup}>
          <p>Are you sure you want to delete this user?</p>
          <button onClick={handleDelete}>Yes, Delete</button>
          <button onClick={() => setShowDeleteConfirm(false)}>Cancel</button>
        </div>
      )}
    </div>
  );
};

export default EditUserDetails;
